@@ -22,12 +22,13 @@
 #define CGDATA_HPP
 
 #include "SparseMatrix.hpp"
+#include "Vector.hpp"
 
 struct CGData_STRUCT {
-  double * r; //!< pointer to residual vector
-  double * z; //!< pointer to preconditioned residual vector
-  double * p; //!< pointer to direction vector
-  double * Ap; //!< pointer to Krylov vector
+  Vector r; //!< pointer to residual vector
+  Vector z; //!< pointer to preconditioned residual vector
+  Vector p; //!< pointer to direction vector
+  Vector Ap; //!< pointer to Krylov vector
 };
 typedef struct CGData_STRUCT CGData;
 
@@ -40,10 +41,10 @@ typedef struct CGData_STRUCT CGData;
 inline void InitializeSparseCGData(SparseMatrix & A, CGData & data) {
   local_int_t nrow = A.localNumberOfRows;
   local_int_t ncol = A.localNumberOfColumns;
-  data.r = new double [nrow]; // Residual vector
-  data.z = new double [nrow]; // Preconditioned residual vector
-  data.Ap = new double [nrow];
-  data.p = new double [ncol]; // Direction vector (in MPI mode ncol>=nrow)
+  InitializeVector(data.r, nrow);
+  InitializeVector(data.z, ncol);
+  InitializeVector(data.p, ncol);
+  InitializeVector(data.Ap, nrow);
   return;
 }
 
@@ -54,10 +55,10 @@ inline void InitializeSparseCGData(SparseMatrix & A, CGData & data) {
  */
 inline void DeleteCGData(CGData & data) {
 
-  delete [] data.r;
-  delete [] data.z;
-  delete [] data.Ap;
-  delete [] data.p;
+  DeleteVector (data.r);
+  DeleteVector (data.z);
+  DeleteVector (data.p);
+  DeleteVector (data.Ap);
   return;
 }
 
