@@ -14,9 +14,9 @@
 
 #ifndef COMPUTESPMV_HPP
 #define COMPUTESPMV_HPP
-#include "Geometry.hpp"
+#include "Vector.hpp"
 #include "SparseMatrix.hpp"
 
-int ComputeSPMV( const SparseMatrix & A, const double * const x, double * const y);
+int ComputeSPMV( const SparseMatrix & A, Vector & x, Vector & y);
 
 #endif  // COMPUTESPMV_HPP
