@@ -14,8 +14,7 @@
 
 #ifndef COMPUTEWAXPBY_HPP
 #define COMPUTEWAXPBY_HPP
-#include "Geometry.hpp"
-int ComputeWAXPBY(const local_int_t n, const double alpha, const double * const x,
-    const double beta, const double * const y,
-    double * const w, bool & isOptimized);
+#include "Vector.hpp"
+int ComputeWAXPBY(const local_int_t n, const double alpha, const Vector & x,
+    const double beta, const Vector & y, Vector & w, bool & isOptimized);
 #endif // COMPUTEWAXPBY_HPP
