@@ -21,6 +21,10 @@
 // Main routine of a program that calls the HPCG conjugate gradient
 // solver to solve the problem, and then prints results.
 
+#ifndef HPCG_NOMPI
+#include <mpi.h> // If this routine is not compiled with HPCG_NOMPI
+#endif
+
 #include <fstream>
 #include <iostream>
 #include <cstdlib>
@@ -33,12 +37,9 @@ using std::endl;
 
 #include "hpcg.hpp"
 
-#ifndef HPCG_NOMPI
-#include <mpi.h> // If this routine is not compiled with HPCG_NOMPI
-#endif
-
 #include "GenerateGeometry.hpp"
 #include "GenerateProblem.hpp"
+#include "GenerateCoarseProblem.hpp"
 #include "SetupHalo.hpp"
 #include "ExchangeHalo.hpp"
 #include "OptimizeProblem.hpp"
@@ -46,12 +47,13 @@ using std::endl;
 #include "ReportResults.hpp"
 #include "mytimer.hpp"
 #include "ComputeSPMV_ref.hpp"
-#include "ComputeSYMGS_ref.hpp"
+#include "ComputeMG_ref.hpp"
 #include "ComputeResidual.hpp"
 #include "CG.hpp"
 #include "CG_ref.hpp"
 #include "Geometry.hpp"
 #include "SparseMatrix.hpp"
+#include "Vector.hpp"
 #include "CGData.hpp"
 #include "TestCG.hpp"
 #include "TestSymmetry.hpp"
@@ -82,9 +84,9 @@ int main(int argc, char * argv[]) {
   if (size < 100 && rank==0) HPCG_fout << "Process "<<rank<<" of "<<size<<" is alive with " << params.numThreads << " threads." <<endl;
 
   if (rank==0) {
-    int junk = 0;
-    HPCG_fout << "Press enter to continue"<< endl;
-    cin >> junk;
+    char c;
+    std::cout << "Press key to continue"<< std::endl;
+    std::cin.get(c);
   }
 #ifndef HPCG_NOMPI
   MPI_Barrier(MPI_COMM_WORLD);
@@ -106,16 +108,24 @@ int main(int argc, char * argv[]) {
 #endif
 
   // Construct the geometry and linear system
-  Geometry geom;
+  Geometry * geom = new Geometry;
   GenerateGeometry(size, rank, params.numThreads, nx, ny, nz, geom);
 
   SparseMatrix A;
-  InitializeSparseMatrix(A);
+  InitializeSparseMatrix(A, geom);
+
+  Vector b, x, xexact;
+  GenerateProblem(A, &b, &x, &xexact);
+  SetupHalo(A);
+  int numberOfMgLevels = 4; // Number of levels including first
+  SparseMatrix * curLevelMatrix = &A;
+  for (int level = 1; level< numberOfMgLevels; ++level) {
+	  GenerateCoarseProblem(*curLevelMatrix);
+	  curLevelMatrix = curLevelMatrix->Ac; // Make the just-constructed coarse grid the next level
+  }
+
 
   CGData data;
-  double * b, *x, *xexact;
-  GenerateProblem(geom, A, &b, &x, &xexact);
-  SetupHalo(geom, A);
   InitializeSparseCGData(A, data);
 
 
@@ -123,14 +133,14 @@ int main(int argc, char * argv[]) {
   std::vector< double > times(9,0.0);
 
   // Call user-tunable set up function.
-  double t7 = mytimer(); OptimizeProblem(geom, A, data, b, x, xexact); t7 = mytimer() - t7;
+  double t7 = mytimer(); OptimizeProblem(A, data, b, x, xexact); t7 = mytimer() - t7;
   times[7] = t7;
 #ifdef HPCG_DEBUG
   if (rank==0) HPCG_fout << "Total problem setup time in main (sec) = " << mytimer() - t1 << endl;
 #endif
 
 #ifdef HPCG_DETAILED_DEBUG
-  if (geom.size==1) WriteProblem(geom, A, b, x, xexact);
+  if (geom.size==1) WriteProblem(A, b, x, xexact);
 #endif
 
 
@@ -143,10 +153,10 @@ int main(int argc, char * argv[]) {
 #endif
   TestCGData testcg_data;
   testcg_data.count_pass = testcg_data.count_fail = 0;
-  TestCG(geom, A, data, b, x, &testcg_data);
+  TestCG(A, data, b, x, testcg_data);
 
   TestSymmetryData testsymmetry_data;
-  TestSymmetry(geom, A, b, xexact, &testsymmetry_data);
+  TestSymmetry(A, b, xexact, testsymmetry_data);
 
 #ifdef HPCG_DEBUG
   if (rank==0) HPCG_fout << "Total validation (TestCG and TestSymmetry) execution time in main (sec) = " << mytimer() - t1 << endl;
@@ -157,39 +167,34 @@ int main(int argc, char * argv[]) {
 #endif
 
   ///////////////////////////////////////
-  // Reference SpMV+SymGS Timing Phase //
+  // Reference SpMV+MG Timing Phase //
   ///////////////////////////////////////
 
-  // Call Reference SpMV and SYMGS. Compute Optimization time as ratio of times in these routines
+  // Call Reference SpMV and MG. Compute Optimization time as ratio of times in these routines
 
   local_int_t nrow = A.localNumberOfRows;
   local_int_t ncol = A.localNumberOfColumns;
 
-  double * x_overlap = new double [ncol]; // Overlapped copy of x vector
-  double * b_computed = new double [nrow]; // Computed RHS vector
+  Vector x_overlap, b_computed;
+  InitializeVector(x_overlap, ncol); // Overlapped copy of x vector
+  InitializeVector(b_computed, nrow); // Computed RHS vector
 
 
-  // Record execution time of reference SpMV and SymGS kernels for reporting times
+  // Record execution time of reference SpMV and MG kernels for reporting times
   // First load vector with random values
-  for (int i=0; i<nrow; ++i) {
-    x_overlap[i] = ((double) rand() / (RAND_MAX)) + 1;
-  }
+  FillRandomVector(x_overlap);
 
   int numberOfCalls = 10;
   double t_begin = mytimer();
   for (int i=0; i< numberOfCalls; ++i) {
-#ifndef HPCG_NOMPI
-    ExchangeHalo(A,x_overlap);
-#endif
     ierr = ComputeSPMV_ref(A, x_overlap, b_computed); // b_computed = A*x_overlap
     if (ierr) HPCG_fout << "Error in call to SpMV: " << ierr << ".\n" << endl;
-    ierr = ComputeSYMGS_ref(A, x_overlap, b_computed); // b_computed = Minv*y_overlap
-    if (ierr) HPCG_fout << "Error in call to SymGS: " << ierr << ".\n" << endl;
+    ierr = ComputeMG_ref(A, b_computed, x_overlap); // b_computed = Minv*y_overlap
+    if (ierr) HPCG_fout << "Error in call to MG: " << ierr << ".\n" << endl;
   }
   times[8] = (mytimer() - t_begin)/((double) numberOfCalls);  // Total time divided by number of calls.
-
 #ifdef HPCG_DEBUG
-  if (rank==0) HPCG_fout << "Total SpMV+SymGS timing phase execution time in main (sec) = " << mytimer() - t1 << endl;
+  if (rank==0) HPCG_fout << "Total SpMV+MG timing phase execution time in main (sec) = " << mytimer() - t1 << endl;
 #endif
 
   ///////////////////////////////
@@ -205,21 +210,21 @@ int main(int argc, char * argv[]) {
   int totalNiters = 0;
   double normr = 0.0;
   double normr0 = 0.0;
-  int maxIters = 50;
+  int refMaxIters = 50;
   numberOfCalls = 1; // Only need to run the residual reduction analysis once
 
   // Compute the residual reduction for the natural ordering and reference kernels
   std::vector< double > ref_times(9,0.0);
-  double tolerance = 0.0; // Set tolerance to zero to make all runs do max_iter iterations
+  double tolerance = 0.0; // Set tolerance to zero to make all runs do maxIters iterations
   int err_count = 0;
   for (int i=0; i< numberOfCalls; ++i) {
-    for (int j=0; j< A.localNumberOfRows; ++j) x[j] = 0.0; // start x at all zeros
-    ierr = CG_ref( geom, A, data, b, x, maxIters, tolerance, niters, normr, normr0, &ref_times[0], true);
+    ZeroVector(x);
+    ierr = CG_ref( A, data, b, x, refMaxIters, tolerance, niters, normr, normr0, &ref_times[0], true);
     if (ierr) ++err_count; // count the number of errors in CG
     totalNiters += niters;
   }
   if (rank == 0 && err_count) HPCG_fout << err_count << " error(s) in call(s) to reference CG." << endl;
-  double ref_tolerance = normr / normr0;
+  double refTolerance = normr / normr0;
 
   //////////////////////////////
   // Optimized CG Setup Phase //
@@ -232,22 +237,22 @@ int main(int argc, char * argv[]) {
   err_count = 0;
   int tolerance_failures = 0;
 
-  int opt_maxIters = 10*maxIters;
-  int opt_iters = 0;
+  int optMaxIters = 10*refMaxIters;
+  int optNiters = 0;
   double opt_worst_time = 0.0;
 
   std::vector< double > opt_times(9,0.0);
 
   // Compute the residual reduction and residual count for the user ordering and optimized kernels.
   for (int i=0; i< numberOfCalls; ++i) {
-    for (int j=0; j< A.localNumberOfRows; ++j) x[j] = 0.0; // start x at all zeros
+    ZeroVector(x); // start x at all zeros
     double last_cummulative_time = opt_times[0];
-    ierr = CG( geom, A, data, b, x, opt_maxIters, ref_tolerance, niters, normr, normr0, &opt_times[0], true);
+    ierr = CG( A, data, b, x, optMaxIters, refTolerance, niters, normr, normr0, &opt_times[0], true);
     if (ierr) ++err_count; // count the number of errors in CG
-    if (normr / normr0 > ref_tolerance) ++tolerance_failures; // the number of failures to reduce residual
+    if (normr / normr0 > refTolerance) ++tolerance_failures; // the number of failures to reduce residual
 
     // pick the largest number of iterations to guarantee convergence
-    if (niters > opt_iters) opt_iters = niters;
+    if (niters > optNiters) optNiters = niters;
 
     double current_time = opt_times[0] - last_cummulative_time;
     if (current_time > opt_worst_time) opt_worst_time = current_time;
@@ -291,13 +296,15 @@ int main(int argc, char * argv[]) {
   /* This is the timed run for a specified amount of time. */
 
   totalNiters = 0;
+  optMaxIters = optNiters;
+  double optTolerance = 0.0;  // Force optMaxIters iterations
   TestNormsData testnorms_data;
   testnorms_data.samples = numberOfCgSets;
   testnorms_data.values = new double[numberOfCgSets];
 
   for (int i=0; i< numberOfCgSets; ++i) {
-    for (int j=0; j< A.localNumberOfRows; ++j) x[j] = 0.0; // Zero out x
-    ierr = CG( geom, A, data, b, x, maxIters, tolerance, niters, normr, normr0, &times[0], true);
+    ZeroVector(x); // Zero out x
+    ierr = CG( A, data, b, x, optMaxIters, optTolerance, niters, normr, normr0, &times[0], true);
     if (ierr) HPCG_fout << "Error in call to CG: " << ierr << ".\n" << endl;
     if (rank==0) HPCG_fout << "Call [" << i << "] Scaled Residual [" << normr/normr0 << "]" << endl;
     testnorms_data.values[i] = normr/normr0; // Record scaled residual from this run
@@ -314,24 +321,24 @@ int main(int argc, char * argv[]) {
 #endif
 
   // Test Norm Results
-  ierr = TestNorms(&testnorms_data);
+  ierr = TestNorms(testnorms_data);
 
   ////////////////////
   // Report Results //
   ////////////////////
 
   // Report results to YAML file
-  ReportResults(geom, A, numberOfCgSets, totalNiters, &times[0], &testcg_data, &testsymmetry_data, &testnorms_data, global_failure);
+  ReportResults(A, numberOfMgLevels, numberOfCgSets, refMaxIters, totalNiters, &times[0], testcg_data, testsymmetry_data, testnorms_data, global_failure);
 
   // Clean up
-  DeleteMatrix(A);
+  DeleteMatrix(A); // This delete will recursively delete all coarse grid data
   DeleteCGData(data);
+  DeleteVector(x);
+  DeleteVector(b);
+  DeleteVector(xexact);
+  DeleteVector(x_overlap);
+  DeleteVector(b_computed);
   delete [] testnorms_data.values;
-  delete [] x;
-  delete [] b;
-  delete [] xexact;
-  delete [] x_overlap;
-  delete [] b_computed;
 
 
 
